"""This module is for the translation mapping data of the
following document:

Document file: unbind_enter_frame_and_unbind_enter_frame_all.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# unbind_enter_frame and unbind_enter_frame_all interfaces": "# unbind_enter_frame と unbind_enter_frame_all の各インターフェイス",  # noqa
    ##################################################
    "This page explains the `unbind_enter_frame` and `unbind_enter_frame_all` methods interfaces.": "このページでは`unbind_enter_frame`と`unbind_enter_frame_all`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `unbind_enter_frame` and `unbind_enter_frame_all` methods disable an enter frame event's handler setting(s).": "`unbind_enter_frame`と`unbind_enter_frame_all`メソッドの各インターフェイスはenter frameイベントのハンドラ設定を取り除きます。",  # noqa
    ##################################################
    "The `unbind_enter_frame` interface disables a specified single handler, and the `unbind_enter_frame_all` interface disables all handlers.": "`unbind_enter_frame`インターフェイスは指定された一つのハンドラを無効化し、`unbind_enter_frame_all`インターフェイスはすべてのハンドラ設定を無効化します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `unbind_enter_frame` interface requires the `handler` argument.": "`unbind_enter_frame`インターフェイスは`handler`引数の指定を必要とします。",  # noqa
    ##################################################
    "In addition, this interface raises an exception if a specified `handler` is not registered yet.": "加えて、このインターフェイスはもし指定されたハンドラがまだ未設定だった場合にはエラーとなります。",  # noqa
    ##################################################
    "In the following example, if you click the rectangle, the handler disables an enter frame event.": "以下の例では、四角をクリックした際にenter frameのイベントを無効化しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\n\n\ndef on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle.rotation_around_center += 1\n\n\ndef on_rectangle_click(\n    e: ap.MouseEvent[ap.Rectangle],\n    options: dict,\n) -> None:\n    """\n    The handler to handle a rectangle click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    stage.unbind_enter_frame(handler=on_enter_frame)\n\n\nstage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\nrectangle.click(handler=on_rectangle_click)\nap.save_overall_html(dest_dir_path="unbind_enter_frame_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\n\n\ndef on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle.rotation_around_center += 1\n\n\ndef on_rectangle_click(\n    e: ap.MouseEvent[ap.Rectangle],\n    options: dict,\n) -> None:\n    """\n    The handler to handle a rectangle click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    stage.unbind_enter_frame(handler=on_enter_frame)\n\n\nstage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\nrectangle.click(handler=on_rectangle_click)\nap.save_overall_html(dest_dir_path="unbind_enter_frame_basic_usage/")\n```',  # noqa
    ##################################################
    "The `unbind_enter_frame_all` interface requires no argument.": "`unbind_enter_frame_all`インターフェイスは引数の指定を必要としません。",  # noqa
    ##################################################
    "In the following example, if you click any rectangle, the handler disables all enter frame events.": "以下の例では四角をクリックした際にすべてのenter frameのイベントを無効化しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nleft_rectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\nright_rectangle: ap.Rectangle = ap.Rectangle(\n    x=150,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#f0a"),\n)\n\n\ndef on_enter_frame_1(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    left_rectangle.rotation_around_center += 1\n\n\ndef on_enter_frame_2(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    right_rectangle.rotation_around_center -= 1\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler to handle a rectangle click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    stage.unbind_enter_frame_all()\n\n\nstage.enter_frame(handler=on_enter_frame_1, fps=ap.FPS.FPS_30)\nstage.enter_frame(handler=on_enter_frame_2, fps=ap.FPS.FPS_30)\nleft_rectangle.click(handler=on_rectangle_click)\nright_rectangle.click(handler=on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="unbind_enter_frame_all_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nleft_rectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\nright_rectangle: ap.Rectangle = ap.Rectangle(\n    x=150,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#f0a"),\n)\n\n\ndef on_enter_frame_1(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    left_rectangle.rotation_around_center += 1\n\n\ndef on_enter_frame_2(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    right_rectangle.rotation_around_center -= 1\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler to handle a rectangle click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    stage.unbind_enter_frame_all()\n\n\nstage.enter_frame(handler=on_enter_frame_1, fps=ap.FPS.FPS_30)\nstage.enter_frame(handler=on_enter_frame_2, fps=ap.FPS.FPS_30)\nleft_rectangle.click(handler=on_rectangle_click)\nright_rectangle.click(handler=on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="unbind_enter_frame_all_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [enter_frame interface](enter_frame.md)": "- [enter_frame インターフェイス](jp_enter_frame.md)",  # noqa
    ##################################################
    "## unbind_enter_frame API": "## unbind_enter_frame のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind a specified handler's enter-frame event.<hr>": "指定されたハンドラのenter-frameイベントの設定を解除します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: Callable[[EnterFrameEvent, _Options], None]": "- `handler`: Callable[[EnterFrameEvent, _Options], None]",  # noqa
    ##################################################
    "  - Unbinding target callable.": "  - 設定を取り除く対象のcallableオブジェクト。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Raises]**": "**[エラー発生条件]**",
    ##################################################
    "- _EnterFrameEventNotRegistered: If there is no unbinding target of a specified handler.": "- _EnterFrameEventNotRegistered: もし指定されたハンドラの設定削除対象が存在しない場合。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n>>> # Any implementations here...\n>>> stage.unbind_enter_frame(handler=on_enter_frame)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n>>> # Any implementations here...\n>>> stage.unbind_enter_frame(handler=on_enter_frame)\n```',  # noqa
    ##################################################
    "## unbind_enter_frame_all API": "## unbind_enter_frame_all のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all enter-frame events.<hr>": "すべてのenter-frameイベントの設定を解除します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n>>> # Any implementations here...\n>>> stage.unbind_enter_frame_all()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n>>> # Any implementations here...\n>>> stage.unbind_enter_frame_all()\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array_last_value.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class last_value property": "# Array クラスの last_value プロパティ",
    ##################################################
    "This page explains the `Array` class `last_value` property.": "このページでは`Array`クラスの`last_value`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `last_value` property returns the last value of an array.": "`last_value`属性は配列の最後の値を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example shows that the `last_value` property becomes 30 (the array values are `[10, 20, 30]`).": "以下の例では`last_value`属性が30になっていることを試しています（配列の値は`[10, 20, 30]`となっています）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[int] = ap.Array([10, 20, 30])\nlast_value: int = arr.last_value\nap.assert_equal(last_value, 30)\n\nap.save_overall_html(dest_dir_path="array_last_value_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[int] = ap.Array([10, 20, 30])\nlast_value: int = arr.last_value\nap.assert_equal(last_value, 30)\n\nap.save_overall_html(dest_dir_path="array_last_value_basic_usage_1/")\n```',  # noqa
    ##################################################
    "If an array's value is empty, this property becomes the `undefined` value on the JavaScript runtime:": "もしも配列の値が空の場合、この属性の値はJavaScriptのランタイム上では`undefined`になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[ap.Int] = ap.Array([], fixed_value_type=ap.Int)\nlast_value: ap.Int = arr.last_value\nap.assert_undefined(last_value)\n\nap.save_overall_html(dest_dir_path="array_last_value_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[ap.Int] = ap.Array([], fixed_value_type=ap.Int)\nlast_value: ap.Int = arr.last_value\nap.assert_undefined(last_value)\n\nap.save_overall_html(dest_dir_path="array_last_value_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## last_value property API": "## last_value 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an array's last index value.<hr>": "配列の最後のインデックスの値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `last_value`: _ArrValue": "- `last_value`: _ArrValue",
    ##################################################
    "  - An array's last index value.": "  - 配列の最後のインデックスの値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    " ・The constructor's `fixed_value_type` setting affects this property's value type. ": " ・コンストラクタの`fixed_value_type`設定はこの属性の値の型に影響します。",  # noqa
    ##################################################
    "<br> ・If an array is empty, this value becomes `undefined` on the JavaScript runtime.<hr>": "<br> ・もし配列が空の場合、この値はJavaScriptのランタイム上では`undefined`となります。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage(\n...     stage_width=100,\n...     stage_height=50,\n...     background_color=ap.Color("#333"),\n...     stage_elem_id="stage",\n... )\n>>> arr: ap.Array[ap.Int] = ap.Array([], fixed_value_type=ap.Int)\n>>> last_value: ap.Int = arr.last_value\n>>> ap.assert_undefined(last_value)\n>>> arr.append(ap.Int(10))\n>>> last_value = arr.last_value\n>>> ap.assert_equal(last_value, 10)\n>>> arr.append(ap.Int(20))\n>>> last_value = arr.last_value\n>>> ap.assert_equal(last_value, 20)\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage(\n...     stage_width=100,\n...     stage_height=50,\n...     background_color=ap.Color("#333"),\n...     stage_elem_id="stage",\n... )\n>>> arr: ap.Array[ap.Int] = ap.Array([], fixed_value_type=ap.Int)\n>>> last_value: ap.Int = arr.last_value\n>>> ap.assert_undefined(last_value)\n>>> arr.append(ap.Int(10))\n>>> last_value = arr.last_value\n>>> ap.assert_equal(last_value, 10)\n>>> arr.append(ap.Int(20))\n>>> last_value = arr.last_value\n>>> ap.assert_equal(last_value, 20)\n```',  # noqa
}



"""Class implementation for line color interface.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._animation.animation_line_color_mixin import AnimationLineColorMixIn
from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class LineColorMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    AnimationLineColorMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    _line_color: Color

    @property
    @add_debug_info_setting(module_name=__name__)
    def line_color(self) -> Color:
        """
        Get this instance's line color.

        Returns
        -------
        line_color : Color
            Current line color (hexadecimal string, e.g., '#00aaff').
            If it is not set, it returns the `COLORLESS` constant.

        References
        ----------
        - GraphicsBase line_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50,
        ...     y=50,
        ...     width=50,
        ...     height=50,
        ...     line_color=ap.Color("#0af"),
        ...     line_thickness=2,
        ... )
        >>> rectangle.line_color
        Color("#00aaff")
        >>> rectangle.line_color = ap.Color("#ff00aa")
        >>> rectangle.line_color
        Color("#ff00aa")
        """
        from apysc._color.color import Color
        from apysc._type import value_util

        self._initialize_line_color_if_not_initialized()
        line_color: Color = value_util.get_copy(value=self._line_color)
        return line_color

    @line_color.setter
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def line_color(self, value: Color) -> None:
        """
        Update this instance's line color.

        Parameters
        ----------
        value : Color
            Line color to set.

        References
        ----------
        - GraphicsBase line_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50,
        ...     y=50,
        ...     width=50,
        ...     height=50,
        ...     line_color=ap.Color("#0af"),
        ...     line_thickness=2,
        ... )
        >>> rectangle.line_color
        Color("#00aaff")
        >>> rectangle.line_color = ap.Color("#ff00aa")
        >>> rectangle.line_color
        Color("#ff00aa")
        """
        value = value.copy()
        self._initialize_line_color_if_not_initialized()
        self._update_line_color_and_skip_appending_exp(value=value)
        self._append_line_color_update_expression()

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_color, attr_name="line_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_color, attr_name="line_color"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_color_update_expression(self) -> None:
        """
        Append line color updating expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{self.variable_name}.stroke({self._line_color._value.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _set_initial_line_color_if_not_colorless(self, *, line_color: Color) -> None:
        """
        Set initial line color value if a specified value is
        not the `COLORLESS` constant.

        Parameters
        ----------
        line_color : Color
            A line color.
        """

        self._initialize_line_color_if_not_initialized()
        if line_color == COLORLESS:
            return
        self._update_line_color_and_skip_appending_exp(value=line_color)

    @final
    def _update_line_color_and_skip_appending_exp(self, *, value: Color) -> None:
        """
        Update line color and skip appending expression.

        Parameters
        ----------
        value : Color
            Line color to set.
        """
        self._initialize_line_color_if_not_initialized()
        self._line_color = value

    @final
    def _initialize_line_color_if_not_initialized(self) -> None:
        """
        Initialize the line_color attribute if this
        interface does not initialize it yet.
        """
        if hasattr(self, "_line_color"):
            return
        self._line_color = COLORLESS

    _line_color_snapshots: Optional[Dict[str, str]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_color_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_line_color_snapshots",
            value=self._line_color._value._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_color_if_not_initialized()
        self._line_color._value._value = self._get_snapshot_val_if_exists(
            current_value=self._line_color._value._value,
            snapshot_dict=self._line_color_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the `mobile_screen_share` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMobileScreenShareIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `mobile_screen_share` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M17 1.01L7 1c-1.1 0-1.99.9-1.99 2v18c0 1.1.89 2 1.99 2h10c1.1 0 2-.9 2-2V3c0-1.1-.9-1.99-2-1.99zM17 19H7V5h10v14zm-4.2-5.78v1.75l3.2-2.99L12.8 9v1.7c-3.11.43-4.35 2.56-4.8 4.7 1.11-1.5 2.58-2.18 4.8-2.18z"/></svg>'  # noqa



"""Class implementation for line joints mix-in.
"""

from typing import Dict
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._display.line_joints import LineJoints
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class LineJointsMixIn(VariableNameSuffixAttrOrVarMixIn, VariableNameMixIn, RevertMixIn):
    _line_joints: String

    @final
    def _initialize_line_joints_if_not_initialized(self) -> None:
        """
        Initialize _line_joints attribute if this interface does
        not initialize it yet.
        """
        if hasattr(self, "_line_joints"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="line_joints"
        )
        self._line_joints = String(
            LineJoints.MITER.value,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def line_joints(self) -> Union[String, LineJoints]:
        """
        Get this instance's line joints style setting.

        Returns
        -------
        line_joints : String
            Line joints style setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)
        >>> _ = sprite.graphics.move_to(x=50, y=100)
        >>> _ = sprite.graphics.line_to(x=100, y=50)
        >>> line: ap.Polyline = sprite.graphics.line_to(x=150, y=100)
        >>> line.line_joints = ap.LineJoints.ROUND
        >>> line.line_joints
        String("round")
        """
        self._initialize_line_joints_if_not_initialized()
        return self._line_joints.copy()

    @line_joints.setter
    @arg_validation_decos.are_line_joints(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def line_joints(self, value: Union[String, LineJoints]) -> None:
        """
        Set line joints style setting.

        Parameters
        ----------
        value : String or LineJoints
            Line joints style setting to set.
        """
        self._update_line_joints_and_skip_appending_exp(value=value)
        self._append_line_joints_update_expression()

    @final
    def _update_line_joints_and_skip_appending_exp(
        self, *, value: Union[String, LineJoints]
    ) -> None:
        """
        Update line joints and skip appending expression.

        Parameters
        ----------
        value : STring or LineJoints
            Line joints style setting to set.
        """
        if not isinstance(value, (String, LineJoints)):
            raise TypeError(
                "Not supported line_joints type specified: "
                f"{type(value)}"
                "\nAcceptable ones are: String or LineJoints."
            )
        if isinstance(value, String):
            self._line_joints = value.copy()
        else:
            self._line_joints = String(value.value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_joints_update_expression(self) -> None:
        """
        Append line cap updating expression.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        joints_name: str = value_util.get_value_str_for_expression(
            value=self._line_joints
        )
        expression: str = (
            f"{self.variable_name}.attr" f'({{"stroke-linejoin": {joints_name}}});'
        )
        expression_data_util.append_js_expression(expression=expression)

    _line_joints_snapshots: Optional[Dict[str, str]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_joints_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_line_joints_snapshots",
            value=self._line_joints._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_joints_if_not_initialized()
        self._line_joints._value = self._get_snapshot_val_if_exists(
            current_value=self._line_joints._value,
            snapshot_dict=self._line_joints_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the `rounded_corner` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRoundedCornerIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `rounded_corner` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M19,19h2v2h-2V19z M19,17h2v-2h-2V17z M3,13h2v-2H3V13z M3,17h2v-2H3V17z M3,9h2V7H3V9z M3,5h2V3H3V5z M7,5h2V3H7V5z M15,21h2v-2h-2V21z M11,21h2v-2h-2V21z M15,21h2v-2h-2V21z M7,21h2v-2H7V21z M3,21h2v-2H3V21z M21,8c0-2.76-2.24-5-5-5h-5v2h5 c1.65,0,3,1.35,3,3v5h2V8z"/></g></g></g></svg>'  # noqa



"""Implementation for the SVG mask class.
"""

from apysc._display.css_mixin import CssMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class SvgMask(
    CssMixIn,
    SetOverflowVisibleSettingMixIn,
):
    """
    The class for the object masking.

    References
    ----------
    - SvgMask class and its related interfaces
        - https://simon-ritchie.github.io/apysc/en/svg_mask.html

    Examples
    --------
    >>> import apysc as ap

    >>> _ = ap.Stage()
    >>> mask: ap.SvgMask = ap.SvgMask()
    >>> circle: ap.Circle = ap.Circle(
    ...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF
    ... )
    >>> mask.add_svg_masking_object(masking_object=circle)
    >>> rectangle: ap.Rectangle = ap.Rectangle(
    ...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF
    ... )
    >>> rectangle.svg_mask = mask
    """

    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    def __init__(
        self,
        *,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The class for the SVG masking.

        Parameters
        ----------
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - SvgMask class and its related interfaces
            - https://simon-ritchie.github.io/apysc/en/svg_mask.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> mask: ap.SvgMask = ap.SvgMask()
        >>> circle: ap.Circle = ap.Circle(
        ...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> mask.add_svg_masking_object(masking_object=circle)
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> rectangle.svg_mask = mask
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.MASK
        )
        self.variable_name = variable_name
        self._append_constructor_expression()
        self._set_overflow_visible_setting()

    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        stage: Stage = get_stage()
        expression: str = f"var {self.variable_name} = {stage.variable_name}.clip();"
        expression_data_util.append_js_expression(expression=expression)

    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_fill_color_mixin(arg_position_index=1)
    def add_svg_masking_object(
        self,
        *,
        masking_object: GraphicsBase,
    ) -> None:
        """
        Add an SVG masking object to this mask. This instance uses its masking object
        to mask other SVG graphics objects.

        It is possible to add multiple masking objects to a mask.

        Parameters
        ----------
        masking_object : GraphicsBase
            The masking object to add.

        References
        ----------
        - SvgMask class and its related interfaces
            - https://simon-ritchie.github.io/apysc/en/svg_mask.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> mask: ap.SvgMask = ap.SvgMask()
        >>> circle: ap.Circle = ap.Circle(
        ...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> mask.add_svg_masking_object(masking_object=circle)
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> rectangle.svg_mask = mask
        """
        from apysc._expression import expression_data_util

        expression: str = f"{self.variable_name}.add({masking_object.variable_name});"
        expression_data_util.append_js_expression(expression=expression)



"""The conversion utility to convert a built-in list of str to
an apysc's Array of String.
"""

from typing import List
from typing import Optional
from typing import Union

from apysc._type.array import Array
from apysc._type.string import String


def list_of_strs_to_array_of_string(
    *,
    optional_list_or_arr: Optional[Union[List[str], Array[String]]],
    variable_name_suffix: str = "",
) -> Optional[Array[String]]:
    """
    Convert a built-in list of str to an apysc's `Array` of `String`.

    Parameters
    ----------
    optional_list_or_arr : Optional[Union[List[str], Array[String]]]
        An optional list of str or array of `String` to convert.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    converted_value : Optional[Array[String]]
        A converted array of `String` value.
        If the `optional_list_or_arr` is None, this function returns None.
    """
    if optional_list_or_arr is None:
        return None
    if isinstance(optional_list_or_arr, list):
        converted_value: Array[String] = Array(
            [
                String(str_value, variable_name_suffix=variable_name_suffix)
                for str_value in optional_list_or_arr
            ],
            variable_name_suffix=variable_name_suffix,
        )
        return converted_value
    return optional_list_or_arr



"""The class implementation for the `app_blocking` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAppBlockingIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `app_blocking` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M18 8c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4-1.79-4-4-4zm-2.5 4c0-1.38 1.12-2.5 2.5-2.5.42 0 .8.11 1.15.29l-3.36 3.36c-.18-.35-.29-.73-.29-1.15zm2.5 2.5c-.42 0-.8-.11-1.15-.29l3.36-3.36c.18.35.29.73.29 1.15 0 1.38-1.12 2.5-2.5 2.5zM17 18H7V6h10v1h2V3c0-1.1-.9-2-2-2H7c-1.1 0-2 .9-2 2v18c0 1.1.9 2 2 2h10c1.1 0 2-.9 2-2v-4h-2v1z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_scale_from_center.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase scale_x_from_center and scale_y_from_center interfaces": "# GraphicsBase クラスの scale_x_from_center と scale_y_from_center インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class (base class of each graphic, such as the `Rectangle`) `scale_x_from_center` and `scale_y_from_center` property interfaces.": "このページでは`GraphicsBase`クラス（`Rectangle`などの各グラフィッククラスの基底クラス）の`scale_x_from_center`と`scale_y_from_center`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `scale_x_from_center` property changes the object's horizontal scale, and the `scale_y_from_center` property changes the object's vertical scale. These scaling interfaces change the scale from the center coordinates of each object.": "`scale_x_from_center`属性はオブジェクトの水平方向の拡縮を変更し、`scale_y_from_center`属性は垂直方向の拡縮を変更します。これらの拡縮のインターフェイスはオブジェクトの中央座標を基準に実行されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each property getter interface returns a `Number` value. The setter interfaces also require a `Number` to update scales (If 0.0 is specified, the object becomes invisible. 1.0 becomes the default scale, and 2.0 becomes the twice-scale value).": "各属性のgetterのインターフェイスは`Number`型の値を返却します。setterのインターフェイスでは拡縮の更新値として`Number`型の値の指定が必要になります（もしも0.0が指定されればオブジェクトは見えなくなり、1.0でデフォルトの拡縮、2.0で2倍のサイズになります）。",  # noqa
    ##################################################
    "The following example shows the default scale rectangle (left), horizontally half-scaled rectangle (center), vertically half-scaled rectangle (right).": "以下のコード例では左の四角ではデフォルトの拡縮値、真ん中の四角では水平方向に半分のサイズ、→の四角では垂直方向に半分のサイズを設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\ncenter_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\ncenter_rectangle.scale_x_from_center = ap.Number(0.5)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50\n)\nright_rectangle.scale_y_from_center = ap.Number(0.5)\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\ncenter_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\ncenter_rectangle.scale_x_from_center = ap.Number(0.5)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50\n)\nright_rectangle.scale_y_from_center = ap.Number(0.5)\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_1/")\n```',  # noqa
    ##################################################
    "These interfaces apply the scaling from the center coordinates as follows:": "これらのインターフェイスでは以下の例のように中央座標を基準に拡縮が実行されます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.3)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2.scale_x_from_center = ap.Number(0.5)\nrectangle_2.scale_y_from_center = ap.Number(0.5)\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_3.scale_x_from_center = ap.Number(0.25)\nrectangle_3.scale_y_from_center = ap.Number(0.25)\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.3)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2.scale_x_from_center = ap.Number(0.5)\nrectangle_2.scale_y_from_center = ap.Number(0.5)\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_3.scale_x_from_center = ap.Number(0.25)\nrectangle_3.scale_y_from_center = ap.Number(0.25)\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_2/")\n```',  # noqa
    ##################################################
    "The `+=` and `-=` operators are also supported:": "`+=`や`-=`記号のオペレーターによる操作もサポートしています:",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectanglesOptions(TypedDict):\n    rectangle_1: ap.Rectangle\n    rectangle_2: ap.Rectangle\n    direction: ap.Int\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectanglesOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = options["rectangle_1"]\n    rectangle_2: ap.Rectangle = options["rectangle_2"]\n    direction: ap.Int = options["direction"]\n\n    current_scale: ap.Number = rectangle_1.scale_x_from_center\n    condition_1: ap.Boolean = current_scale >= 2.0\n    condition_2: ap.Boolean = current_scale <= 0.05\n    with ap.If(condition_1):\n        direction.value = -1\n    with ap.Elif(condition_2):\n        direction.value = 1\n\n    rectangle_1.scale_x_from_center += direction * 0.03\n    rectangle_2.scale_y_from_center += direction * 0.03\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color=ap.Color("#f0a"), alpha=0.5)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\ndirection: ap.Int = ap.Int(1.0)\noptions: _RectanglesOptions = {\n    "rectangle_1": rectangle_1,\n    "rectangle_2": rectangle_2,\n    "direction": direction,\n}\ntimer: ap.Timer = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_3/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectanglesOptions(TypedDict):\n    rectangle_1: ap.Rectangle\n    rectangle_2: ap.Rectangle\n    direction: ap.Int\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectanglesOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = options["rectangle_1"]\n    rectangle_2: ap.Rectangle = options["rectangle_2"]\n    direction: ap.Int = options["direction"]\n\n    current_scale: ap.Number = rectangle_1.scale_x_from_center\n    condition_1: ap.Boolean = current_scale >= 2.0\n    condition_2: ap.Boolean = current_scale <= 0.05\n    with ap.If(condition_1):\n        direction.value = -1\n    with ap.Elif(condition_2):\n        direction.value = 1\n\n    rectangle_1.scale_x_from_center += direction * 0.03\n    rectangle_2.scale_y_from_center += direction * 0.03\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color=ap.Color("#f0a"), alpha=0.5)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\ndirection: ap.Int = ap.Int(1.0)\noptions: _RectanglesOptions = {\n    "rectangle_1": rectangle_1,\n    "rectangle_2": rectangle_2,\n    "direction": direction,\n}\ntimer: ap.Timer = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="graphics_base_scale_from_center_basic_usage_3/")\n```',  # noqa
    ##################################################
    "## scale_x_from_center property API": "## scale_x_from_center 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a scale-x value from the center of this instance.<hr>": "インスタンスの中央座標を基準とした水平方向の拡縮の値を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `scale_x_from_center`: ap.Number": "- `scale_x_from_center`: ap.Number",
    ##################################################
    "  - Scale-x value from the center of this instance.": "  - インスタンスの中央座標を基準とした水平方向の拡縮の値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "The scale's minimum value is almost zero, and it does not become negative.<hr>": "拡縮の最小値はほぼ0となり、その値は負の値にはなりません。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.scale_x_from_center = ap.Number(1.5)\n>>> rectangle.scale_x_from_center\nNumber(1.5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.scale_x_from_center = ap.Number(1.5)\n>>> rectangle.scale_x_from_center\nNumber(1.5)\n```',  # noqa
    ##################################################
    "## scale_y_from_center property API": "## scale_y_from_center 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a scale-y value from the center of this instance.<hr>": "インスタンスの中央座標を基準とした垂直方向の拡縮の値を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `scale_y_from_center`: ap.Number": "- `scale_y_from_center`: ap.Number",
    ##################################################
    "  - Scale-y value from the center of this instance.": "  - インスタンスの中央座標を基準とした垂直方向の拡縮の値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "The scale's minimum value is almost zero, and it does not become negative.<hr>": "拡縮の最小値はほぼ0となり、その値は負の値にはなりません。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.scale_y_from_center = ap.Number(1.5)\n>>> rectangle.scale_y_from_center\nNumber(1.5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.scale_y_from_center = ap.Number(1.5)\n>>> rectangle.scale_y_from_center\nNumber(1.5)\n```',  # noqa
}



"""The class implementation for the `notification_important` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialNotificationImportantIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `notification_important` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M18 16v-5c0-3.07-1.64-5.64-4.5-6.32V4c0-.83-.67-1.5-1.5-1.5s-1.5.67-1.5 1.5v.68C7.63 5.36 6 7.92 6 11v5l-2 2v1h16v-1l-2-2zm-5 0h-2v-2h2v2zm0-4h-2V8h2v4zm-1 10c1.1 0 2-.9 2-2h-4c0 1.1.89 2 2 2z"/></svg>'  # noqa



"""The class implementation for the outlined `contacts` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `contacts` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M20 4H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h16c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zm0 14H4V6h16v12zM4 0h16v2H4zm0 22h16v2H4zm8-10c1.38 0 2.5-1.12 2.5-2.5S13.38 7 12 7 9.5 8.12 9.5 9.5 10.62 12 12 12zm0-3.5c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm5 7.49C17 13.9 13.69 13 12 13s-5 .9-5 2.99V17h10v-1.01zm-8.19-.49c.61-.52 2.03-1 3.19-1 1.17 0 2.59.48 3.2 1H8.81z"/></svg>'  # noqa



"""The class implementation for the outlined `mic` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMicOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mic` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><rect fill="none" height="24" width="24"/><rect fill="none" height="24" width="24"/></g><g><g/><g><path d="M12,14c1.66,0,3-1.34,3-3V5c0-1.66-1.34-3-3-3S9,3.34,9,5v6C9,12.66,10.34,14,12,14z"/><path d="M17,11c0,2.76-2.24,5-5,5s-5-2.24-5-5H5c0,3.53,2.61,6.43,6,6.92V21h2v-3.08c3.39-0.49,6-3.39,6-6.92H17z"/></g></g></svg>'  # noqa



"""The class implementation for the `copyright` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCopyrightIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `copyright` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M11.88,9.14c1.28,0.06,1.61,1.15,1.63,1.66h1.79c-0.08-1.98-1.49-3.19-3.45-3.19C9.64,7.61,8,9,8,12.14 c0,1.94,0.93,4.24,3.84,4.24c2.22,0,3.41-1.65,3.44-2.95h-1.79c-0.03,0.59-0.45,1.38-1.63,1.44C10.55,14.83,10,13.81,10,12.14 C10,9.25,11.28,9.16,11.88,9.14z M12,2C6.48,2,2,6.48,2,12s4.48,10,10,10s10-4.48,10-10S17.52,2,12,2z M12,20c-4.41,0-8-3.59-8-8 s3.59-8,8-8s8,3.59,8,8S16.41,20,12,20z"/></g></g></g></svg>'  # noqa



"""The class implementation for the `outlet` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialOutletIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `outlet` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M12,2C6.48,2,2,6.48,2,12c0,5.52,4.48,10,10,10s10-4.48,10-10C22,6.48,17.52,2,12,2z M9,12c-0.55,0-1-0.45-1-1V8 c0-0.55,0.45-1,1-1s1,0.45,1,1v3C10,11.55,9.55,12,9,12z M14,18h-4v-2c0-1.1,0.9-2,2-2c1.1,0,2,0.9,2,2V18z M16,11 c0,0.55-0.45,1-1,1c-0.55,0-1-0.45-1-1V8c0-0.55,0.45-1,1-1c0.55,0,1,0.45,1,1V11z"/></svg>'  # noqa



"""The class implementation for the outlined `high_quality` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHighQualityOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `high_quality` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 4H5c-1.11 0-2 .9-2 2v12c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zm0 14H5V6h14v12zM7.5 13h2v2H11V9H9.5v2.5h-2V9H6v6h1.5zm6.5 2h.75v1.5h1.5V15H17c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm.5-4.5h2v3h-2v-3z"/></svg>'  # noqa



"""Class Implementation for an event.
"""

from typing import Generic
from typing import Optional
from typing import TypeVar

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos

_Target = TypeVar("_Target", bound=VariableNameMixIn)


class Event(Generic[_Target], VariableNameMixIn):
    """
    Basic event class.

    Examples
    --------
    >>> import apysc as ap
    >>> def on_custom_event(e: ap.Event[ap.Rectangle], options: dict) -> None:
    ...     rectangle: ap.Rectangle = e.this
    ...     rectangle.fill_color = ap.Color("#f0a")
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> e: ap.Event = ap.Event(this=rectangle)
    >>> _ = rectangle.bind_custom_event(
    ...     custom_event_type="my_custom_event", handler=on_custom_event, e=e
    ... )
    >>> # Do something here and then trigger the custom event
    >>> rectangle.trigger_custom_event(custom_event_type="my_custom_event")
    """

    _this: _Target

    @arg_validation_decos.is_variable_name_interface_type(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=True)
    @add_debug_info_setting(module_name=__name__)
    def __init__(self, *, this: _Target, type_name: Optional[str] = None) -> None:
        """
        Basic event class.

        Parameters
        ----------
        this : VariableNameMixIn
            Instance that listening event (e.g., Sprite).
        type_name : str or None, default None
            Type name to set. Only specify when inheriting
            this class.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.Color("#f0a")
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type="my_custom_event", handler=on_custom_event, e=e
        ... )
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(custom_event_type="my_custom_event")
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._validate_type_name_and_self_type(type_name=type_name)
        self._this = this
        if type_name is None:
            type_name = var_names.EVENT
        self.variable_name = expression_variables_util.get_next_variable_name(
            type_name=type_name
        )

    @final
    def _validate_type_name_and_self_type(self, *, type_name: Optional[str]) -> None:
        """
        Validate type_name argument is None when a self
        instance is not Event subclass, and the same is true
        for the opposite pattern.

        Parameters
        ----------
        type_name : str or None
            Type name to set.

        Raises
        ------
        ValueError
            - If type_name is not None and self instance is Event type.
            - If type_name is None and self instance is not Event type.
        """
        from apysc._type import type_util

        if type_name is not None:
            if type_util.is_same_class_instance(class_=Event, instance=self):
                raise ValueError(
                    "type_name argument can be set only when this instance "
                    "is subclass of Event."
                )
            return
        if not type_util.is_same_class_instance(class_=Event, instance=self):
            raise ValueError(
                "type_name argument can't be set when this instance "
                "is Event (this will be used by Event subclass)."
            )

    @property
    def this(self) -> _Target:
        """
        Get an instance of listening to this event.

        Returns
        -------
        this : VariableNameMixIn
            Instance that listening this event.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.Color("#f0a")
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type="my_custom_event", handler=on_custom_event, e=e
        ... )
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(custom_event_type="my_custom_event")
        """
        return self._this



"""The mix-in class implementation for the `SvgText`'s `_set_leading` method.
"""

from typing import Union
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class SvgTextSetLeadingMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_leading(self, *, leading: Union[float, Number]) -> None:
        """
        Set a leading value.

        Parameters
        ----------
        leading : Union[float, Number]
            A text-leading value.
        """
        from apysc._display.svg_text_leading_mixin import SvgTextLeadingMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if not isinstance(self, SvgTextLeadingMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextLeadingMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if isinstance(leading, float):
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self, value_identifier="leading"
            )
            leading_: Number = Number(leading, variable_name_suffix=suffix)
        else:
            leading_ = cast(Number, leading)
        self.leading = leading_



"""The class implementation for the outlined `build` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBuildOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `build` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M22.61 18.99l-9.08-9.08c.93-2.34.45-5.1-1.44-7C9.79.61 6.21.4 3.66 2.26L7.5 6.11 6.08 7.52 2.25 3.69C.39 6.23.6 9.82 2.9 12.11c1.86 1.86 4.57 2.35 6.89 1.48l9.11 9.11c.39.39 1.02.39 1.41 0l2.3-2.3c.4-.38.4-1.01 0-1.41zm-3 1.6l-9.46-9.46c-.61.45-1.29.72-2 .82-1.36.2-2.79-.21-3.83-1.25C3.37 9.76 2.93 8.5 3 7.26l3.09 3.09 4.24-4.24-3.09-3.09c1.24-.07 2.49.37 3.44 1.31 1.08 1.08 1.49 2.57 1.24 3.96-.12.71-.42 1.37-.88 1.96l9.45 9.45-.88.89z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: animation_pause_and_play.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_pause and animation_play interfaces": "# animation_pause と animation_play インターフェイス",  # noqa
    ##################################################
    "This page explains the `animation_pause` and `animation_play` method interfaces.": "このページでは`animation_pause`と`animation_play`のメソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface are these?": "## 各インターフェイス概要",
    ##################################################
    "The `animation_pause` interface pauses all the running animations of the target instance. The `animation_play` interface restarts the paused animation.": "`animation_pause`インターフェイスは対象のインスタンスで動いている全てのアニメーションを停止します。`animation_play`インターフェイスは停止しているアニメーションを再開します。",  # noqa
    ##################################################
    "These interfaces exist in the instances that have the animation interfaces (such as the `animation_x`\\, `animation_move`).": "これらのインターフェイスは`animation_x`や`animation_move`などのアニメーション関係のインターフェイスを持つクラスのインスタンス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example starts the x-coordinate animation and stops after 1 second. After stopping the animation and additionally, 500 milliseconds passed, restarting the animation:": "以下のコード例ではX座標のアニメーションをスタートし、その1秒後にアニメーションを停止しています。加えて、停止してからさらに500ミリ秒経過した時点でアニメーションを再開しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer_1(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : _RectOptions\n        Optional arguments dictionary.\n    """\n    options["rectangle"].animation_pause()\n    timer: ap.Timer = ap.Timer(on_timer_2, delay=500, options=options, repeat_count=1)\n    timer.start()\n\n\ndef on_timer_2(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : _RectOptions\n        Optional arguments dictionary.\n    """\n    options["rectangle"].animation_play()\n\n\nap.Stage(\n    stage_width=600,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=500, duration=15_000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(on_timer_1, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="./animation_pause_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer_1(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : _RectOptions\n        Optional arguments dictionary.\n    """\n    options["rectangle"].animation_pause()\n    timer: ap.Timer = ap.Timer(on_timer_2, delay=500, options=options, repeat_count=1)\n    timer.start()\n\n\ndef on_timer_2(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : _RectOptions\n        Optional arguments dictionary.\n    """\n    options["rectangle"].animation_play()\n\n\nap.Stage(\n    stage_width=600,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=500, duration=15_000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(on_timer_1, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="./animation_pause_basic_usage/")\n```',  # noqa
    ##################################################
    "## animation_pause API": "## animation_pause API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Stop all animations.<hr>": "全てのアニメーションを停止します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_pause()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_pause()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```',  # noqa
    ##################################################
    "## animation_play API": "## animation_play API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Restart all paused animations.<hr>": "停止している全てのアニメーションを再開します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer_1(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_pause()\n>>> def on_timer_2(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_play()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer_1, delay=500, options=options).start()\n>>> ap.Timer(on_timer_2, delay=1000, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer_1(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_pause()\n>>> def on_timer_2(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_play()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer_1, delay=500, options=options).start()\n>>> ap.Timer(on_timer_2, delay=1000, options=options).start()\n```',  # noqa
}



"""Class implementation for begin_fill method-related mix-in.
"""

from typing import Dict
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class BeginFillMixIn(VariableNameSuffixAttrOrVarMixIn, RevertMixIn):
    _fill_color: Color
    _fill_alpha: Number

    @final
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def begin_fill(self, *, color: Color, alpha: Union[float, Number] = 1.0) -> None:
        """
        Set single color value for fill.

        Parameters
        ----------
        color : Color
            A color setting.
        alpha : float or Number, default 1.0
            Color opacity (0.0 to 1.0).

        References
        ----------
        - Graphics begin_fill interface
            - https://simon-ritchie.github.io/apysc/en/graphics_begin_fill.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(
        ...     color=ap.Color("#0af"),
        ...     alpha=0.5,
        ... )
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.fill_color
        Color("#00aaff")
        >>> rectangle.fill_alpha
        Number(0.5)
        """
        from apysc._converter import cast

        self._initialize_fill_color_if_not_initialized()
        self._initialize_fill_alpha_if_not_initialized()
        self._fill_color = color
        if not isinstance(alpha, Number):
            alpha = cast.to_float_from_int(int_or_float=alpha)
        if isinstance(alpha, Number):
            self._fill_alpha.value = alpha.value
        else:
            self._fill_alpha.value = alpha

    @property
    @add_debug_info_setting(module_name=__name__)
    def fill_color(self) -> Color:
        """
        Get a current fill color.

        Returns
        -------
        fill_color : Color
            Current fill-color.
            If not be set, this interface returns a `COLORLESS` constant.

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(
        ...     color=ap.Color("#0af"),
        ...     alpha=0.5,
        ... )
        >>> sprite.graphics.fill_color
        Color("#00aaff")
        """

        self._initialize_fill_color_if_not_initialized()
        fill_color: Color = self._fill_color.copy()
        return fill_color

    @final
    def _initialize_fill_color_if_not_initialized(self) -> None:
        """
        Initialize the fill_color attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, "_fill_color"):
            return
        self._fill_color = COLORLESS

    @property
    @add_debug_info_setting(module_name=__name__)
    def fill_alpha(self) -> Number:
        """
        Get current fill color opacity.

        Returns
        -------
        fill_alpha : Number
            Current fill color opacity (0.0 to 1.0).
            If not be set, 1.0 will be returned.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.fill_alpha
        Number(0.5)
        """
        from apysc._type import value_util

        self._initialize_fill_alpha_if_not_initialized()
        fill_alpha: Number = value_util.get_copy(value=self._fill_alpha)
        return fill_alpha

    @final
    def _initialize_fill_alpha_if_not_initialized(self) -> None:
        """
        Initialize the fill_alpha attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, "_fill_alpha"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="fill_alpha"
        )
        self._fill_alpha = Number(
            1.0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    _fill_color_snapshots: Optional[Dict[str, str]] = None
    _fill_alpha_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_fill_color_if_not_initialized()
        self._initialize_fill_alpha_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_fill_color_snapshots",
            value=self._fill_color._value._value,
            snapshot_name=snapshot_name,
        )
        self._set_single_snapshot_val_to_dict(
            dict_name="_fill_alpha_snapshots",
            value=self._fill_alpha._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._fill_color._value._value = self._get_snapshot_val_if_exists(
            current_value=self._fill_color._value._value,
            snapshot_dict=self._fill_color_snapshots,
            snapshot_name=snapshot_name,
        )
        self._fill_alpha._value = self._get_snapshot_val_if_exists(
            current_value=self._fill_alpha._value,
            snapshot_dict=self._fill_alpha_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the outlined `remove_shopping_cart` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRemoveShoppingCartOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `remove_shopping_cart` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M1.41 1.13L0 2.54l4.39 4.39 2.21 4.66-1.35 2.45c-.16.28-.25.61-.25.96 0 1.1.9 2 2 2h7.46l1.38 1.38c-.5.36-.83.95-.83 1.62 0 1.1.89 2 1.99 2 .67 0 1.26-.33 1.62-.84L21.46 24l1.41-1.41L1.41 1.13zM7 15l1.1-2h2.36l2 2H7zM20 4H7.12l2 2h9.19l-2.76 5h-1.44l1.94 1.94c.54-.14.99-.49 1.25-.97l3.58-6.49C21.25 4.82 20.76 4 20 4zM7 18c-1.1 0-1.99.9-1.99 2S5.9 22 7 22s2-.9 2-2-.9-2-2-2z"/></svg>'  # noqa



"""The mix-in class for the `from_rgb` method.
"""

from typing import TYPE_CHECKING
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._color.color import Color


class FromRgbMixIn:
    @final
    @classmethod
    @arg_validation_decos.is_uint8_range(arg_position_index=1)
    @arg_validation_decos.is_uint8_range(arg_position_index=2)
    @arg_validation_decos.is_uint8_range(arg_position_index=3)
    @arg_validation_decos.is_builtin_string(arg_position_index=4, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def from_rgb(
        cls,
        *,
        red: Union[int, Int],
        green: Union[int, Int],
        blue: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "Color":
        """
        Create a color instance from RGB (red, green, and blue) values.

        Parameters
        ----------
        red : Union[int, Int]
            A red color value (0 to 255).
        green : Union[int, Int]
            A green color value (0 to 255).
        blue : Union[int, Int]
            A blue color value (0 to 255).
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        color : Color
            A created color instance.

        References
        ----------
        - Color class from_rgb class method
            - https://simon-ritchie.github.io/apysc/en/color_from_rgb.html
        - Color class
            - https://simon-ritchie.github.io/apysc/en/color.html

        Examples
        --------
        >>> import apysc as ap
        >>> color: ap.Color = ap.Color.from_rgb(red=0, green=255, blue=0)
        >>> color
        Color("#00FF00")
        """
        from apysc._color import color_util
        from apysc._color.color import Color
        from apysc._expression import expression_data_util
        from apysc._type.string import String

        color: Color = Color("", variable_name_suffix=variable_name_suffix)
        color._value._value = _get_py_str_from_rgb(red=red, green=green, blue=blue)
        red_hex_str: String = color_util.get_hex_apysc_string_from_int(
            color_int=red, variable_name_suffix=variable_name_suffix
        )
        green_hex_str: String = color_util.get_hex_apysc_string_from_int(
            color_int=green, variable_name_suffix=variable_name_suffix
        )
        blue_hex_str: String = color_util.get_hex_apysc_string_from_int(
            color_int=blue, variable_name_suffix=variable_name_suffix
        )
        color._value._value = (
            f"#{red_hex_str._value}{green_hex_str._value}{blue_hex_str._value}"
        )
        expression: str = (
            f'{color._value.variable_name} = "#" + {red_hex_str.variable_name}'
            f" + {green_hex_str.variable_name} + {blue_hex_str.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)
        return color


def _get_py_str_from_rgb(
    *,
    red: Union[int, Int],
    green: Union[int, Int],
    blue: Union[int, Int],
) -> str:
    """
    Get a Python string from RGB values.

    Parameters
    ----------
    red : Union[int, Int]
        The red color.
    green : Union[int, Int]
        The green color.
    blue : Union[int, Int]
        The blue color.

    Returns
    -------
    py_str : str
        A Python string, e.g., '#0000ff'.
    """
    from apysc._color import color_util

    red_hex: str = color_util.get_hex_str_from_int(color_int=red)
    green_hex: str = color_util.get_hex_str_from_int(color_int=green)
    blue_hex: str = color_util.get_hex_str_from_int(color_int=blue)
    return f"#{red_hex}{green_hex}{blue_hex}"



"""This module is for the translation mapping data of the
following document:

Document file: datetime_set_month_end.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DateTime class set_month_end interface": "# DateTime クラスの set_month_end インターフェイス",  # noqa
    ##################################################
    "This page explains the `DateTime` class's `set_month_end` method interface.": "このページでは`DateTime`クラスの`set_month_end`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `set_month_end` method interface sets the current month's end date.": "`set_month_end`メソッドのインターフェイスでは現在の月の月末の日付を設定します。",  # noqa
    ##################################################
    "For instance, if the current date is 05/12/2022, this method sets 31/12/2022.": "例えば、もし現在の日付が2022-12-05であればこのメソッドによって2022-12-31が設定されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "This method interface requires no arguments.": "このメソッドでは引数を必要としません。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\ndatetime_.set_month_end()\nassert datetime_.day == 31\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)\ndatetime_.set_month_end()\nassert datetime_.day == 31\n```",  # noqa
}



"""The class implementation for the `book_online` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBookOnlineIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `book_online` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><g><path d="M17,1H7C5.9,1,5,1.9,5,3v18c0,1.1,0.9,2,2,2h10c1.1,0,2-0.9,2-2V3C19,1.9,18.1,1,17,1z M7,18V6h10v12H7z M16,11V9.14 C16,8.51,15.55,8,15,8H9C8.45,8,8,8.51,8,9.14l0,1.96c0.55,0,1,0.45,1,1c0,0.55-0.45,1-1,1l0,1.76C8,15.49,8.45,16,9,16h6 c0.55,0,1-0.51,1-1.14V13c-0.55,0-1-0.45-1-1C15,11.45,15.45,11,16,11z M12.5,14.5h-1v-1h1V14.5z M12.5,12.5h-1v-1h1V12.5z M12.5,10.5h-1v-1h1V10.5z"/></g></svg>'  # noqa



"""String's validation implementations.
"""

from typing import Any
from typing import Union

from apysc._type.string import String


def validate_string_type(
    *, string: Union[str, String], additional_err_msg: str = ""
) -> None:
    """
    Validate a specified string's type is str.

    Parameters
    ----------
    string : String or str
        A string to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If a non-string value is specified.
    """
    from apysc._validation import validation_common_utils

    if isinstance(string, str):
        return
    if isinstance(string, String):
        return
    err_msg: str = f"A specified value is not str type: {type(string)}"
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_builtin_string_type(*, string: str, additional_err_msg: str = "") -> None:
    """
    Validate a specified string's type is Python built-in's str.

    Parameters
    ----------
    string : str
        A string to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If a non-string value is specified.
    """
    from apysc._validation import validation_common_utils

    if isinstance(string, str):
        return
    err_msg: str = f"A specified value is not str type: {type(string)}"
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_apysc_string_type(*, string: Any) -> String:
    """
    Validate whether the specified argument is the apysc's `String`
    type or not.

    Parameters
    ----------
    string : Any
        The target string to check.

    Returns
    -------
    string : String
        An apysc's string.

    Raises
    ------
    TypeError
        If the specified argument is not the apysc's `String` type.
    """
    if isinstance(string, String):
        return string
    raise TypeError(
        "The specified argument is not the apysc's `String` type: "
        f"{type(string).__name__}"
    )


def validate_not_empty_string(
    *, string: Union[str, String], additional_err_msg: str = ""
) -> None:
    """
    Validate whether a specified string is not empty.

    Parameters
    ----------
    string : String or str
        String to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If empty string ('' or "") is specified.
        - If specified value is not str type.
    """
    from apysc._validation import validation_common_utils

    validate_string_type(string=string)
    if isinstance(string, str):
        if string != "":
            return
    elif isinstance(string, String):
        if string._value != "":
            return
    err_msg: str = "Empty string is not acceptable."
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)



"""The mix-in class for the `to_hex` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class ToHexMixIn:
    @final
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def to_hex(self, *, variable_name_suffix: str = "") -> String:
        """
        Get a hexadecimal string (e.g., "1f").

        Notes
        -----
        This method ignores floating point numbers.

        Parameters
        ----------
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        hex_str : String
            A hexadecimal string (e.g., "1f").

        References
        ----------
        - Int and Number classes to_hex method
            - https://simon-ritchie.github.io/apysc/en/int_and_number_to_hex.html

        Examples
        --------
        >>> import apysc as ap
        >>> int_value: ap.Int = ap.Int(28)
        >>> hex_str: ap.String = int_value.to_hex()
        >>> hex_str
        String("1c")

        >>> number: ap.Number = ap.Number(28.5)
        >>> hex_str = int_value.to_hex()
        >>> hex_str
        String("1c")
        """
        from apysc._expression import expression_data_util
        from apysc._math.math import Math
        from apysc._type.int import Int
        from apysc._type.number import Number
        from apysc._validation import number_validation

        int_or_num: Union[Int, Number] = number_validation.validate_apysc_int_or_number(
            value=self
        )
        hex_py_str: str = hex(int(int_or_num._value)).replace("0x", "", 1)
        truncated_int: Int = Math.trunc(int_or_num)
        hex_str: String = String("", variable_name_suffix=variable_name_suffix)
        hex_str._value = hex_py_str
        expression: str = (
            f"{hex_str.variable_name} = {truncated_int.variable_name}.toString(16);"
        )
        expression_data_util.append_js_expression(expression=expression)
        return hex_str



"""The class implementation for the `model_training` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialModelTrainingIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `model_training` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M15.5,13.5c0,2-2.5,3.5-2.5,5h-2c0-1.5-2.5-3-2.5-5c0-1.93,1.57-3.5,3.5-3.5h0C13.93,10,15.5,11.57,15.5,13.5z M13,19.5h-2 V21h2V19.5z M19,13c0,1.68-0.59,3.21-1.58,4.42l1.42,1.42C20.18,17.27,21,15.23,21,13c0-2.74-1.23-5.19-3.16-6.84l-1.42,1.42 C17.99,8.86,19,10.82,19,13z M16,5l-4-4v3c0,0,0,0,0,0c-4.97,0-9,4.03-9,9c0,2.23,0.82,4.27,2.16,5.84l1.42-1.42 C5.59,16.21,5,14.68,5,13c0-3.86,3.14-7,7-7c0,0,0,0,0,0v3L16,5z"/></g></svg>'  # noqa



"""Class implementation for an SVG text.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._color.colors import Colors
from apysc._color.copy_color_if_default_value_specified_mixin import (
    CopyColorIfDefaultValueSpecifiedMixIn,
)
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.append_fill_alpha_attr_expression_mixin import (
    AppendFillAlphaAttrExpressionMixIn,
)
from apysc._display.append_fill_color_expression_mixin import (
    AppendFillColorAttrExpressionMixIn,
)
from apysc._display.append_line_alpha_attr_expression_mixin import (
    AppendLineAlphaAttrExpressionMixIn,
)
from apysc._display.append_line_color_attr_expression_mixin import (
    AppendLineColorAttrExpressionMixIn,
)
from apysc._display.append_line_thickness_attr_expression_mixin import (
    AppendLineThicknessAttrExpressionMixIn,
)
from apysc._display.append_x_attr_expression_mixin import AppendXAttrExpressionMixIn
from apysc._display.append_y_attr_expression_mixin import AppendYAttrExpressionMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.flip_x_mixin import FlipXMixIn
from apysc._display.flip_y_mixin import FlipYMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_thickness_mixin import LineThicknessMixIn
from apysc._display.rotation_around_center_mixin import RotationAroundCenterMixIn
from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
from apysc._display.scale_x_from_center_mixin import ScaleXFromCenterMixIn
from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.skew_x_mixin import SkewXMixIn
from apysc._display.skew_y_mixin import SkewYMixIn
from apysc._display.svg_text_align_mixin import SvgTextAlign
from apysc._display.svg_text_align_mixin import SvgTextAlignMixIn
from apysc._display.svg_text_bold_mixin import SvgTextBoldMixIn
from apysc._display.svg_text_font_family_mixin import SvgTextFontFamilyMixIn
from apysc._display.svg_text_font_size_mixin import SvgTextFontSizeMixIn
from apysc._display.svg_text_italic_mixin import SvgTextItalicMixIn
from apysc._display.svg_text_leading_mixin import SvgTextLeadingMixIn
from apysc._display.svg_text_set_align_mixin import SvgTextSetAlignMixIn
from apysc._display.svg_text_set_bold_mixin import SvgTextSetBoldMixIn
from apysc._display.svg_text_set_font_family_mixin import SvgTextSetFontFamilyMixIn
from apysc._display.svg_text_set_font_size_value_mixin import (
    SvgTextSetFontSizeValueMixIn,
)
from apysc._display.svg_text_set_italic_mixin import SvgTextSetItalicMixIn
from apysc._display.svg_text_set_leading_mixin import SvgTextSetLeadingMixIn
from apysc._display.svg_text_set_text_value_mixin import SvgTextSetTextValueMixIn
from apysc._display.svg_text_span import SvgTextSpan
from apysc._display.svg_text_text_mixin import SvgTextTextMixIn
from apysc._display.use_hand_cursor_mixin import UseHandCursorMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._mask.svg_mask_mixin import SvgMaskMixIn
from apysc._type.array import Array
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.repr_interface import ReprInterface
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class SvgText(
    ReprInterface,
    XMixIn,
    AppendXAttrExpressionMixIn,
    YMixIn,
    AppendYAttrExpressionMixIn,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    GraphicsBase,
    RotationAroundCenterMixIn,
    RotationAroundPointMixIn,
    ScaleXFromCenterMixIn,
    ScaleXFromPointMixIn,
    FlipXMixIn,
    FlipYMixIn,
    SkewXMixIn,
    SkewYMixIn,
    FillColorMixIn,
    AppendFillColorAttrExpressionMixIn,
    FillAlphaMixIn,
    AppendFillAlphaAttrExpressionMixIn,
    LineColorMixIn,
    AppendLineColorAttrExpressionMixIn,
    LineAlphaMixIn,
    AppendLineAlphaAttrExpressionMixIn,
    AppendLineThicknessAttrExpressionMixIn,
    LineThicknessMixIn,
    SvgTextTextMixIn,
    SvgTextSetTextValueMixIn,
    SvgTextFontFamilyMixIn,
    SvgTextSetFontFamilyMixIn,
    SvgTextFontSizeMixIn,
    SvgTextSetFontSizeValueMixIn,
    SvgTextLeadingMixIn,
    SvgTextSetLeadingMixIn,
    SvgTextAlignMixIn,
    SvgTextSetAlignMixIn,
    SvgTextItalicMixIn,
    SvgTextSetItalicMixIn,
    SvgTextBoldMixIn,
    SvgTextSetBoldMixIn,
    GetBoundsMixIn,
    UseHandCursorMixIn,
    VariableNameSuffixMixIn,
    InitializeWithBaseValueInterface,
    CopyColorIfDefaultValueSpecifiedMixIn,
    AddToParentMixIn,
    SvgMaskMixIn,
):
    """
    The class for an SVG text.

    Notes
    -----
    - SvgText's y-coordinate zero-position starts at the bottom of a text.
        So if you set y=0, a text becomes almost invisible.

    References
    ----------
    - SvgText class
        - https://simon-ritchie.github.io/apysc/en/svg_text.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage(
    ...     background_color=ap.Color("#333"),
    ...     stage_width=200,
    ...     stage_height=50,
    ... )
    >>> svg_text: ap.SvgText = ap.SvgText(
    ...     text="Hello, world!",
    ...     font_size=20,
    ...     fill_color=ap.Color("#0af"),
    ... )
    >>> svg_text.text
    String("Hello, world!")
    >>> svg_text.font_size
    Int(20)
    >>> svg_text.fill_color
    Color("#00aaff")
    """

    # text
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    # font_size
    @arg_validation_decos.is_integer(arg_position_index=2, optional=False)
    # font_family
    @arg_validation_decos.is_builtin_str_list_or_apysc_str_arr(
        arg_position_index=3, optional=True
    )
    # x
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    # y
    @arg_validation_decos.is_num(arg_position_index=5, optional=False)
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=6, optional=False)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=7, optional=False)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=8, optional=False)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=9, optional=False)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=10, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=10, optional=False)
    # leading
    @arg_validation_decos.is_num(arg_position_index=11, optional=False)
    # align
    @arg_validation_decos.is_svg_text_align(arg_position_index=12)
    # bold
    @arg_validation_decos.is_boolean(arg_position_index=13, optional=False)
    # italic
    @arg_validation_decos.is_boolean(arg_position_index=14, optional=False)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=15, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=16, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        text: Union[str, String],
        font_size: Union[int, Int] = 16,
        font_family: Optional[Union[Array[String], List[str]]] = None,
        x: Union[float, Number] = 0.0,
        y: Union[float, Number] = 16.0,
        fill_color: Color = Colors.GRAY_666666,
        fill_alpha: Union[float, Number] = 1.0,
        line_color: Color = COLORLESS,
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        leading: Union[float, Number] = 1.5,
        align: SvgTextAlign = SvgTextAlign.LEFT,
        bold: Union[bool, Boolean] = False,
        italic: Union[bool, Boolean] = False,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The class for an SVG text.

        Notes
        -----
        - SvgText's y-coordinate zero-position starts at the bottom of a text.
            So if you set y=0, a text becomes almost invisible.

        References
        ----------
        - SvgText class
            - https://simon-ritchie.github.io/apysc/en/svg_text.html

        Parameters
        ----------
        text : Union[str, String]
            A text to use in this class.
        font_size : Union[int, Int], optional
            A font-size setting.
        font_family : Optional[Union[Array[String], List[str]]], optional
            A font-family setting.
            Each string in an array needs to be a font name (e.g., `Times New Roman`).
        x : float or Number, optional
            X-coordinate to start drawing.
        y : float or Number, optional
            Y-coordinate to start drawing (please see also the `Notes` section).
        fill_color : Color
            A fill-color setting.
        fill_alpha : float or Number, optional
            A fill-alpha setting.
        line_color : Color, optional
            A line-color setting.
        line_alpha : float or Number, optional
            A line-alpha setting.
        line_thickness : int or Int, optional
            A line-thickness (line-width) setting.
        leading : float or Number, optional
            A text-leading size.
        align : SvgTextAlign, default SvgTextAlign.LEFT
            A text-align setting.
        bold : Union[bool, Boolean], optional
            A boolean, whether this text is a bold style or not.
        italic : Union[bool, Boolean], optional
            A boolean, whether a text is an italic style or not (normal).
        parent : ChildMixIn or None, optional
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=200,
        ...     stage_height=50,
        ...     stage_elem_id="stage",
        ... )
        >>> svg_text: ap.SvgText = ap.SvgText(
        ...     text="Hello, world!",
        ...     font_size=20,
        ...     fill_color=ap.Color("#0af"),
        ... )
        >>> svg_text.text
        String("Hello, world!")
        >>> svg_text.font_size
        Int(20)
        >>> svg_text.fill_color
        Color("#00aaff")
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        fill_color = self._copy_color_if_default_value_specified(
            color=fill_color,
            default_color=Colors.GRAY_666666,
        )

        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.SVG_TEXT,
        )
        self.variable_name = variable_name
        self._variable_name_suffix = variable_name_suffix
        self._update_x_and_skip_appending_exp(x=x)
        self._update_y_and_skip_appending_exp(y=y)
        self._set_initial_basic_values(
            fill_color=fill_color,
            fill_alpha=fill_alpha,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=None,
            line_joints=None,
        )

        super(SvgText, self).__init__(
            variable_name=variable_name,
        )

        self._append_constructor_expression()
        self._set_text_value(text=text)
        self._set_font_size_value(font_size=font_size)
        self._set_font_family(font_family=font_family)
        self._set_leading(leading=leading)
        self._set_align(align=align)
        self._set_bold(bold=bold)
        self._set_italic(italic=italic)
        self._add_to_parent(parent=parent)
        self._set_overflow_visible_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression string.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        INDENT_NUM: int = 2
        variable_name: str = self.variable_name
        stage: Stage = get_stage()
        expression: str = (
            f"var {variable_name} = {stage.variable_name}" "\n  .text()\n  .attr({"
        )
        expression = self._append_fill_color_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_fill_alpha_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_color_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_thickness_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_alpha_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_x_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_y_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression += "\n  });"
        expression_data_util.append_js_expression(expression=expression)

    @classmethod
    # text_spans
    @arg_validation_decos.are_text_spans(arg_position_index=1)
    # font_size
    @arg_validation_decos.is_integer(arg_position_index=2, optional=False)
    # font_family
    @arg_validation_decos.is_builtin_str_list_or_apysc_str_arr(
        arg_position_index=3, optional=True
    )
    # x
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    # y
    @arg_validation_decos.is_num(arg_position_index=5, optional=False)
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=6, optional=False)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=7, optional=False)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=8, optional=False)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=9, optional=False)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=10, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=10, optional=False)
    # leading
    @arg_validation_decos.is_num(arg_position_index=11, optional=False)
    # align
    @arg_validation_decos.is_svg_text_align(arg_position_index=12)
    # bold
    @arg_validation_decos.is_boolean(arg_position_index=13, optional=False)
    # italic
    @arg_validation_decos.is_boolean(arg_position_index=14, optional=False)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=15, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=16, optional=False)
    def create_with_svg_text_spans(
        cls,
        *,
        text_spans: Union[List[SvgTextSpan], Array[SvgTextSpan]],
        font_size: Union[int, Int] = 16,
        font_family: Optional[Union[Array[String], List[str]]] = None,
        x: Union[float, Number] = 0.0,
        y: Union[float, Number] = 16.0,
        fill_color: Color = Colors.GRAY_666666,
        fill_alpha: Union[float, Number] = 1.0,
        line_color: Color = COLORLESS,
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        leading: Union[float, Number] = 1.5,
        align: SvgTextAlign = SvgTextAlign.LEFT,
        bold: Union[bool, Boolean] = False,
        italic: Union[bool, Boolean] = False,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> "SvgText":
        """
        Create an `SvgText` instance with specified text spans.

        Notes
        -----
        - SvgText's y-coordinate zero-position starts at the bottom of a text.
            So if you set y=0, a text becomes almost invisible.

        References
        ----------
        - SvgText class
            - https://simon-ritchie.github.io/apysc/en/svg_text.html
        - SvgTextSpan class
            - https://simon-ritchie.github.io/apysc/en/svg_text_span.html

        Parameters
        ----------
        text_spans : Union[List[SvgTextSpan], Array[SvgTextSpan]]
            Text spans.
        font_size : Union[int, Int], optional
            A font-size setting for an overall text.
        font_family : Optional[Union[Array[String], List[str]]], optional
            A font-family setting for an overall text.
            Each string in an array needs to be a font name (e.g., `Times New Roman`).
        x : Union[float, Number], optional
            X-coordinate to start drawing.
        y : Union[float, Number], optional
            Y-coordinate to start drawing (please see also the `Notes` section).
        fill_color : Color, optional
            A fill-color setting for an overall text.
        fill_alpha : float or Number, optional
            A fill-alpha setting for an overall text.
        line_color : Color, optional
            A line-color setting for an overall text.
        line_alpha : float or Number, optional
            A line-alpha setting for an overall text.
        line_thickness : int or Int, optional
            A line-thickness (line-width) setting for an overall text.
        leading : float or Number, optional
            A text-leading size for an overall text.
        align : SvgTextAlign, optional
            A text-align setting for an overall text.
        bold : Union[bool, Boolean], optional
            A boolean, whether this text is a bold style or not.
        italic : Union[bool, Boolean], optional
            A boolean, whether a text is an italic style or not (normal).
        parent : Optional[ChildMixIn], optional
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        svg_text : SvgText
            A created `SvgText` instance.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=200,
        ...     stage_height=50,
        ... )
        >>> svg_text: ap.SvgText = ap.SvgText.create_with_svg_text_spans(
        ...     text_spans=[
        ...         ap.SvgTextSpan(text="Hello, "),
        ...         ap.SvgTextSpan(text="Hello, ", font_size=14),
        ...     ],
        ...     font_size=20,
        ...     fill_color=ap.Color("#0af"),
        ... )
        """
        from apysc._expression import expression_data_util
        from apysc._loop.for_array_indices import ForArrayIndices

        svg_text: SvgText = SvgText(
            text="",
            font_size=font_size,
            font_family=font_family,
            x=x,
            y=y,
            fill_color=fill_color,
            fill_alpha=fill_alpha,
            line_color=line_color,
            line_alpha=line_alpha,
            line_thickness=line_thickness,
            leading=leading,
            align=align,
            bold=bold,
            italic=italic,
            parent=parent,
            variable_name_suffix=variable_name_suffix,
        )
        text_spans_: Array[SvgTextSpan] = svg_text._convert_text_spans_list_to_array(
            text_spans=text_spans
        )
        i: Int
        with ForArrayIndices(
            arr=text_spans_,
            locals_=locals(),
            globals_=globals(),
        ) as i:
            text_span: SvgTextSpan = text_spans_[i]
            expression: str = (
                f"{svg_text.variable_name}.add({text_span.variable_name});"
            )
            expression_data_util.append_js_expression(expression=expression)
        return svg_text

    @final
    @add_debug_info_setting(module_name=__name__)
    def _convert_text_spans_list_to_array(
        self,
        *,
        text_spans: Union[List[SvgTextSpan], Array[SvgTextSpan]],
    ) -> Array[SvgTextSpan]:
        """
        Convert text spans' list to an array.

        Parameters
        ----------
        text_spans : Union[List[SvgTextSpan], Array[SvgTextSpan]]
            Text spans.

        Returns
        -------
        text_spans_ : Array[SvgTextSpan]
            A converted array.
        """
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if isinstance(text_spans, Array):
            return text_spans
        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="text_spans",
        )
        text_spans_: Array[SvgTextSpan] = Array(text_spans, variable_name_suffix=suffix)
        return text_spans_

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            This interface returns a type name and variable name
            (e.g., `SvgText("<variable_name>")`).
        """

        repr_str: str = f'{SvgText.__name__}("{self.variable_name}")'
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "SvgText":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        svg_text : SvgText
            An initialized svg text instance.
        """
        svg_text: SvgText = SvgText(text="")
        svg_text.visible = Boolean(False)
        return svg_text



"""Class implementation for fill color mix-in.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._animation.animation_fill_color_mixin import AnimationFillColorMixIn
from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class FillColorMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    AnimationFillColorMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    _fill_color: Color

    @property
    @add_debug_info_setting(module_name=__name__)
    def fill_color(self) -> Color:
        """
        Get this instance's fill color.

        Returns
        -------
        fill_color : Color
            Current fill color.
            If it is not set, it returns the `COLORLESS` constant.

        References
        ----------
        - GraphicsBase fill_color property
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> circle: ap.Circle = ap.Circle(
        ...     x=75,
        ...     y=75,
        ...     radius=50,
        ...     fill_color=ap.Color("#0af"),
        ... )
        >>> circle.fill_color
        Color("#00aaff")

        >>> circle.fill_color = ap.Color("#ff00aa")
        >>> circle.fill_color
        Color("#ff00aa")
        """
        from apysc._type import value_util

        self._initialize_fill_color_if_not_initialized()
        fill_color: Color = value_util.get_copy(value=self._fill_color)
        return fill_color

    @fill_color.setter
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def fill_color(self, value: Color) -> None:
        """
        Update this instance's fill color.

        Parameters
        ----------
        value : Color
            Fill color to set.

        References
        ----------
        - GraphicsBase fill_color property
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     stage_width=150,
        ...     stage_height=150,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )
        >>> circle: ap.Circle = ap.Circle(
        ...     x=75,
        ...     y=75,
        ...     radius=50,
        ...     fill_color=ap.Color("#0af"),
        ... )
        >>> circle.fill_color
        Color("#00aaff")

        >>> circle.fill_color = ap.Color("#ff00aa")
        >>> circle.fill_color
        Color("#ff00aa")
        """
        value = value.copy()
        self._update_fill_color_and_skip_appending_exp(value=value)
        self._append_fill_color_update_expression()

        self._append_applying_new_attr_val_exp(
            new_attr=self._fill_color, attr_name="fill_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._fill_color, attr_name="fill_color"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_fill_color_update_expression(self) -> None:
        """
        Append the fill color updating expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{self.variable_name}.fill({self.fill_color._value.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_fill_color_if_not_colorless(self, *, fill_color: Color) -> None:
        """
        Set the initial fill color if a specified value
        is not the `COLORLESS` constant.

        Parameters
        ----------
        fill_color : Color
            A fill color.
        """

        self._initialize_fill_color_if_not_initialized()
        if fill_color == COLORLESS:
            return
        self._update_fill_color_and_skip_appending_exp(value=fill_color)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _update_fill_color_and_skip_appending_exp(self, *, value: Color) -> None:
        """
        Update fill color and skip appending expression.

        Parameters
        ----------
        value : Color
            Fill color to set.
        """
        self._initialize_fill_color_if_not_initialized()
        self._fill_color = value

    @final
    @add_debug_info_setting(module_name=__name__)
    def _initialize_fill_color_if_not_initialized(self) -> None:
        """
        Initialize the fill_color attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, "_fill_color"):
            return
        self._fill_color = COLORLESS

    _fill_color_snapshots: Optional[Dict[str, str]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_fill_color_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_fill_color_snapshots",
            value=self._fill_color._value._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._fill_color._value._value = self._get_snapshot_val_if_exists(
            current_value=self._fill_color._value._value,
            snapshot_dict=self._fill_color_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the outlined `preview` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPreviewOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `preview` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M19,3H5C3.89,3,3,3.9,3,5v14c0,1.1,0.89,2,2,2h14c1.1,0,2-0.9,2-2V5C21,3.9,20.11,3,19,3z M19,19H5V7h14V19z M12,10.5 c1.84,0,3.48,0.96,4.34,2.5c-0.86,1.54-2.5,2.5-4.34,2.5S8.52,14.54,7.66,13C8.52,11.46,10.16,10.5,12,10.5 M12,9 c-2.73,0-5.06,1.66-6,4c0.94,2.34,3.27,4,6,4s5.06-1.66,6-4C17.06,10.66,14.73,9,12,9L12,9z M12,14.5c-0.83,0-1.5-0.67-1.5-1.5 s0.67-1.5,1.5-1.5s1.5,0.67,1.5,1.5S12.83,14.5,12,14.5z"/></g></svg>'  # noqa



"""The class implementation for the `verified` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVerifiedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `verified` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M23,12l-2.44-2.79l0.34-3.69l-3.61-0.82L15.4,1.5L12,2.96L8.6,1.5L6.71,4.69L3.1,5.5L3.44,9.2L1,12l2.44,2.79l-0.34,3.7 l3.61,0.82L8.6,22.5l3.4-1.47l3.4,1.46l1.89-3.19l3.61-0.82l-0.34-3.69L23,12z M10.09,16.72l-3.8-3.81l1.48-1.48l2.32,2.33 l5.85-5.87l1.48,1.48L10.09,16.72z"/></g></svg>'  # noqa



"""The class implementation for the `biotech` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBiotechIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `biotech` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M7,19c-1.1,0-2,0.9-2,2h14c0-1.1-0.9-2-2-2h-4v-2h3c1.1,0,2-0.9,2-2h-8c-1.66,0-3-1.34-3-3c0-1.09,0.59-2.04,1.46-2.56 C8.17,9.03,8,8.54,8,8c0-0.21,0.04-0.42,0.09-0.62C6.28,8.13,5,9.92,5,12c0,2.76,2.24,5,5,5v2H7z"/><path d="M10.56,5.51C11.91,5.54,13,6.64,13,8c0,0.75-0.33,1.41-0.85,1.87l0.59,1.62l0.94-0.34l0.34,0.94l1.88-0.68l-0.34-0.94 l0.94-0.34L13.76,2.6l-0.94,0.34L12.48,2L10.6,2.68l0.34,0.94L10,3.97L10.56,5.51z"/><circle cx="10.5" cy="8" r="1.5"/></g></g></svg>'  # noqa



"""Class implementation for the SVG text's align mix-in.
"""

from enum import Enum
from typing import Dict
from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class SvgTextAlign(Enum):
    """
    SVG text's align enum class.

    Attributes
    ----------
    LEFT : str
        An enum definition to align left-position.
    CENTER : str
        An enum definition to align center-position.
    RIGHT : str
        An enum definition to align right-position.
    """

    LEFT = "start"
    CENTER = "middle"
    RIGHT = "end"


class SvgTextAlignMixIn(
    VariableNameMixIn,
    RevertMixIn,
):
    _align: SvgTextAlign = SvgTextAlign.LEFT

    @property
    @add_debug_info_setting(module_name=__name__)
    def align(self) -> SvgTextAlign:
        """
        Get a current text-align setting.

        Returns
        -------
        align : SvgTextAlign
            A current text-align setting.
        """
        return self._align

    @align.setter
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_svg_text_align(arg_position_index=1)
    def align(self, value: SvgTextAlign) -> None:
        """
        Set a text-align setting.

        Parameters
        ----------
        value : SvgTextAlign
            A text-align setting.
        """
        from apysc._expression import expression_data_util

        self._align = value
        expression: str = f'{self.variable_name}.font("anchor", "{value.value}");'
        expression_data_util.append_js_expression(expression=expression)

    _align_snapshots: Optional[Dict[str, SvgTextAlign]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_align_snapshots",
            value=self._align,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._align = self._get_snapshot_val_if_exists(
            current_value=self._align,
            snapshot_dict=self._align_snapshots,
            snapshot_name=snapshot_name,
        )
